/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "attributes.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <sys/types.h>
#include <sys/stat.h>

#include "client_msgs.h"
#include "defines.h"
#include "log.h"
#include "misc.h"
#include "objects.h"
#include "strutil.h"

/* local static variables */
static llist_struct *global_attributes       = NULL;
static llist_struct *internal_attribute_list = NULL;

/* -------------------- LOCAL FUNCTIONS ------------------------ */

/* returns how to index the word based on the attribute. If none
     declared, returns all words. */
static a_index_type
select_word_index_type (itype)
  char *itype;
{
  if (STR_EQ(itype, A_INDEX_NONE_WORDS))
  {
    return INDEX_NO_WORDS;
  }
  else if (STR_EQ(itype, A_INDEX_FIRST_WORDS))
  {
    return INDEX_FIRST_WORD;
  }
  else if (STR_EQ(itype, A_INDEX_ONLY_WORDS))
  {
    return INDEX_WORD_ONLY;
  }
  else
  {
    return INDEX_ALL_WORDS;
  }
}

/* returns the description of the attribute */
static a_type
select_type(type)
  char *type;
{
  if (STR_EQ(type, A_SEE_ALSO))
  {
    return (TYPE_SEE_ALSO);
  }
  else if (STR_EQ(type, A_REFERRAL))
  {
    return (TYPE_REFERRAL);
  }
  else if (STR_EQ(type, A_TYPE_FILE))
  {
    return (TYPE_FILE);
  }
  else if (STR_EQ(type, A_TYPE_MIME))
  {
    return (TYPE_MIME);
  }
  else
  {
    return (TYPE_TEXT);
  }
}

/* -------------------- PUBLIC FUNCTIONS ----------------------- */

/* reads the attributes for the class; returns TRUE if ok, FALSE if
   not. */
int
read_attributes (object)
  object_struct *object;
{
  FILE              *fp     = NULL;
  attribute_struct  *top    = NULL;
  attribute_struct  *cur    = NULL;
  attribute_struct  *new    = NULL;
  char              line[MAX_LINE];
  char              tag[MAX_TEMPLATE_NAME];
  char              datum[MAX_TEMPLATE_NAME];
  struct stat       buf;

  /* this may be for the global type */
  if (object->attribute_fname == NULL)
  {
    return TRUE;
  }

  if ((fp = fopen(object->attribute_fname, "r")) == (FILE *)NULL)
  {
    log_error("could not open attribute template '%s': %s",
              object->attribute_fname, l_strerror(errno));
    return FALSE;
  } 

  /* create the version number */
  if (stat(object->attribute_fname,  &buf) < 0)
  {
    log_error("unable to stat schema file '%s': %s",
              object->attribute_fname, l_strerror(errno));
    object->version = 0;
  }
  else
  {
    object->version = buf.st_mtime;
  }

  while ((fgets (line, MAX_LINE, fp)) != NULL)
  {
    stripchar (line, '\n');

    if (new_record (line) || top == NULL) {

      new = NEW_ATTRIBUTE;

      if (top == NULL)
      {
        top = cur = new;
      }
      else
      {
        cur->object_name          = NEW_STRING (object->name);
        cur->global_id            = set_attribute_global_id (cur, object);
        cur->internal_id          = set_internal_id (cur);
        cur->object_references    = append_llist (NULL, object);
        cur->no_object_references = 1;
        cur->nxt                  = new;
        cur                       = new;
      }
    }
    
    if (parse_line (line, tag, datum))
    {
      if (STR_EQ(tag, A_ATTRIBUTE))
      {
        cur->name = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, A_ATTRIB_ALIAS))
      {
        cur->name_alias = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, A_DESCRIPTION))
      {
        cur->description = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, A_DEF_OBJECT))
      {
        cur->def_object = true_false(datum);
      }
      else if (STR_EQ(tag, A_IS_PRIMARY_KEY))
      {
        cur->primary_key = true_false(datum);
      }
      else if (STR_EQ(tag, A_IS_AUTH_AREA))
      {
        cur->authority_area = true_false(datum);
      }
      else if (STR_EQ(tag, A_IS_SOA_SERIAL))
      {
        cur->soa = true_false(datum);
      }
      else if (STR_EQ(tag, A_IS_REQUIRED))
      {
        cur->required = true_false(datum);
      }
      else if (STR_EQ(tag, A_MULT_LINES))
      {
        cur->mult_lines = true_false(datum);
      }
      else if (STR_EQ(tag, A_MULTIPLE_GRP))
      {
        cur->multiple_grp = true_false(datum);
      }
      else if (STR_EQ(tag, A_FORMAT))
      {
        cur->format = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, A_INDEX))
      {
        cur->index_words = select_word_index_type(datum);
      }
      else if (STR_EQ(tag, A_TYPE))
      {
        cur->type = select_type(datum);
      }
      /* else ignore */
    }
  }

  cur->object_name          = NEW_STRING (object->name);
  cur->global_id            = set_attribute_global_id (new, object);
  cur->internal_id          = set_internal_id (cur);
  cur->object_references    = append_llist (NULL, object);
  cur->no_object_references = 1;

  fclose(fp);

  object->attributes = top;

  return TRUE;
}

/* returns the unique number for that particular index */
int
set_attribute_global_id(attrib, object)
  attribute_struct  *attrib;
  object_struct     *object;

{
  llist_struct      *cur;
  attribute_struct  *ptr; 
  int               i = 0;

  cur = global_attributes;

  while (cur != NULL)
  {
    ptr = cur->datum;
    
    if (STR_EQ(ptr->name, attrib->name))
    {
      ptr->no_object_references++;
      ptr->object_references = append_llist(ptr->object_references,
                                            object);
      return i;
    }

    cur = cur->nxt;
    i++;
  }

  global_attributes = append_llist(global_attributes, attrib);

  return i;
}

/* sets a linked list of all the attributes; returns the uniq internal
     id */
int
set_internal_id(attrib)
  attribute_struct *attrib;
{
  llist_struct  *cur    = internal_attribute_list;
  int           id      = 0;

  if (cur == NULL)
  {
    internal_attribute_list = append_llist(NULL, attrib);
    return id;
  }
  
  id = 1;

  while (cur->nxt != NULL)
  {
    cur = cur->nxt;
    id++;
  }

  internal_attribute_list = append_llist(internal_attribute_list, attrib);

  return id;
}

/* returns the description of the attribute */
char *
show_attribute_type(type)
  a_type type;
{
  switch (type)
  {
  case TYPE_SEE_ALSO:
    return (A_SEE_ALSO);
    break;
  case TYPE_REFERRAL:
    return (A_REFERRAL);
    break;
  case TYPE_FILE:
    return (A_TYPE_FILE);
    break;
  case TYPE_MIME:
    return (A_TYPE_MIME);
    break;
  case TYPE_TEXT:
    return (A_TYPE_TEXT);
    break;
  default:
    return ("UNKNOWN");
    break;
  }
}

/* returns the global id of the attribute based on txt, -1 if not
      found */
int
get_attribute_global_id_from_txt(attribute_txt)
  char *attribute_txt;
{
  llist_struct      *cur = global_attributes;
  attribute_struct  *ptr = NULL;
  int               i    = 0;

  while (cur != NULL)
  {
    ptr = cur->datum;
    if (STR_EQ(attribute_txt, ptr->name) || 
        STR_EQ(attribute_txt, ptr->name_alias))
    {
      return (i);
    }

    i++;
    cur = cur->nxt;
  }

  return (-1);
}
 
/* returns the attribute info for the particular global_id, NULL if
     not found */
attribute_struct *
get_attribute_info_from_global_id(offset)
  int offset;
{
  llist_struct  *cur =  global_attributes;
  int           i    =  0;

  while (cur != NULL)
  {
    if (i == offset)
    {
      return (cur->datum);
    }

    i++;
    cur = cur->nxt;
  }

  return (NULL);
}

/* returns the attribute info for the particular global_id, NULL if
     not found */
attribute_struct *
get_attribute_info_from_internal_id(offset)
  int offset;
{
  llist_struct  *cur = internal_attribute_list;
  int           i    = 0;

  while (cur != NULL)
  {
    if (i == offset)
    {
      return (cur->datum);
    }

    i++;
    cur = cur->nxt;
  }

  return (NULL);
}

/* returns the attribute info for the particular attribute and object
      names */
attribute_struct *
get_attribute_info_from_name_and_object(object_name,
                                        attribute_name)
  char *object_name;
  char *attribute_name;
{
  attribute_struct *ptr;

  if ((ptr = get_object_attributes_by_object_name (object_name)) != NULL)
  {
    while (ptr != NULL)
    {
      if (STR_EQ(ptr->name, attribute_name) || 
          (ptr->name_alias != NULL && 
           STR_EQ(ptr->name_alias, attribute_name)))
      {
        return (ptr);
      }

      ptr = ptr->nxt;
    }
  }

  return (NULL);
}

/* returns the global id of the attribute based on txt, -1 if not
      found */
int
get_id_from_object_and_name(object_name, attribute_name)
  char *object_name;
  char *attribute_name;
{
  attribute_struct  *ptr = NULL;

  if ((ptr = get_object_attributes_by_object_name (object_name)) != NULL )
  {
    while (ptr != NULL)
    {
      if (STR_EQ(attribute_name, ptr->name) ||
          (ptr->name_alias != NULL &&
           STR_EQ(attribute_name, ptr->name_alias)))
      {
        return (ptr->internal_id);
      }

      ptr = ptr->nxt;
    }
  }

  return (-1);
}

/* returns the name of the attribute */
char *
get_attribute_name(id)
  int id;
{
  attribute_struct *attrib;
  
  if (id == -1 ||
      (attrib = get_attribute_info_from_internal_id (id)) == NULL)
  {
    return ("");
  }

  return (attrib->name);
}

/* returns the global list of attributes */
llist_struct *
global_list_of_attributes()
{
  return (global_attributes);
}

/* sets the attribute list based on object */
attribute_struct *
set_attributes_list_from_object(mode, tag, datum)
  int   mode;
  char  *tag;
  char  *datum;
{
  static attribute_struct *cur = NULL;

  /* resets the attribute list */
  if (mode == FALSE)
  {
    cur = NULL;
    return (NULL);
  }

  /* if cur is null, finds the attribute list - else returns cur */

  /*   printf("set_attributes_list_from_object: tag |%s| datum |%s|\n", tag, */
  /*         datum); */

  if (cur == NULL)
  {
    return (cur = get_object_attributes_by_tag(tag, datum));
  }
  else
  {
    return (cur);
  }
}

/* finds the specific attribute */
attribute_struct *
find_specific_attribute(attrib, tag)
  attribute_struct  *attrib;
  char              *tag;
{
  while (attrib != NULL)
  {
    if (STR_EQ(tag, attrib->name) ||
        STR_EQ(tag, attrib->name_alias))
    {
      return (attrib);
    }

    attrib=attrib->nxt;
  }

  return (NULL);
}
