/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "read_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "client_msgs.h"
#include "define_conf.h"
#include "defines.h"
#include "directive_conf.h"
#include "display_conf.h"
#include "fileutils.h"
#include "log.h"
#include "misc.h"
#include "objects.h"
#include "variables.h"

/* read_all_config_files: get all the possible configuration */
int
read_all_config_files(config_file, chrooted)
  char  *config_file;
  int   chrooted;
{
  char *define_file;
  char *ext_dir_file;
  
  /* first get the main config file and the schema files */
  if (!read_schema_config_files(config_file, chrooted))
  {
    return FALSE;
  }

  /* this is sort of a hack to get around the enormous confusion of
     path names that exists in this code */
  store_current_wd();
  chdir_root_dir();
  
  /* can't operate with out a display file (yet, anyway) */
  if (!read_displays(get_display_file()))
  {
    return FALSE;
  }

  /* no define file is OK for now */
  define_file = get_define_file();
  if (define_file && *define_file)
  {
    read_define_file(define_file);
  }
  
  /* no extended directive file OK, too */
  ext_dir_file = get_directive_file();
  if (ext_dir_file && *ext_dir_file)
  {
    read_extended_directive_file(ext_dir_file);
  }

  restore_current_wd();
  
  return TRUE;
}

/* read_schema_config_files: get all the config data needed for the schema */
int
read_schema_config_files(config_file, chrooted)
  char  *config_file;
  int   chrooted;
{
  /* read the main config file to get the location of everything */
  if (!read_main_config_file(config_file, chrooted))
  {
    return FALSE;
  }
  else if (! verify_config_data())
  {
    return FALSE;
  }

  /* the hack to get around pathname confusion */
  store_current_wd();
  chdir_root_dir();
  
  /* read the schema file to load the entire schema */
  if (!read_objects(get_schema_file()))
  {
    return FALSE;
  }

  restore_current_wd();
  
  return TRUE;
}

/* read_main_config_file: read the main configuration file.  Does not
      take care of second pass validation (for now). */
int
read_main_config_file(config_file, chrooted)
  char  *config_file;
  int   chrooted;
{
  FILE  *fp;
  char  line[BUFSIZ];
  char  tag[MAX_TEMPLATE_DESC];
  char  datum[MAX_TEMPLATE_DESC];
  int   fatal                       = FALSE;

  /* attempt to open the configuration file */
  if ((fp = fopen(config_file, "r")) == NULL)
  {
    log_error("could not open config file '%s': %s", config_file,
              l_strerror(errno));
    return FALSE;
  }

  while ((fgets (line, BUFSIZ, fp)) != NULL && !fatal)
  {
    /* parse line will automatically skip commented lines and invalid
       (no ':' lines */
    if (parse_line(line, tag, datum))
    {
      if (STR_EQ(tag, I_DEFAULT_DIR) || STR_EQ(tag, I_ROOT_DIR))
      {
        /* if we are already chrooted (we must trust the flag), then
           we must assume that the root dir is "/", so we don't need
           to set it again */
        if (!chrooted)
        {
          fatal = (! set_root_dir(datum));
        }
        else
        {
          set_root_dir("/");
        }
      }
      else if (STR_EQ(tag, I_BIN_LOC) || STR_EQ(tag, I_BIN_PATH))
      {
        set_bin_path(datum);
      }
      else if (STR_EQ(tag, I_NOTIFY_LOG))
      {
        set_notify_log(datum);
      }
      else if (STR_EQ(tag, I_DEFINE_FILE))
      {
        set_define_file(datum);
      }
      else if (STR_EQ(tag, I_DIRECTIVE_FILE))
      {
        set_directive_file(datum);
      }
      else if (STR_EQ(tag, I_DISPLAY_FILE))
      {
        set_display_file(datum);
      }
      else if (STR_EQ(tag, I_OBJECT_FILE) || STR_EQ(tag, I_SCHEMA_FILE))
      {
        set_schema_file(datum);
      }
      else if (STR_EQ(tag, I_REGISTER_LOG))
      {
        set_register_log(datum);
      }
      else if (STR_EQ(tag, I_REGISTER_SPOOL))
      {
        set_register_spool(datum);
      }
      else if (STR_EQ(tag, I_UPDATES_DIR))
      {
        set_updates_dir(datum);
      }
      else if (STR_EQ(tag, I_ROOT_FILE))
      {
        set_root_file(datum);
      }
      else if (STR_EQ(tag, I_SECURITY_ALLOW))
      {
        set_security_allow(datum);
      }
      else if (STR_EQ(tag, I_SECURITY_DENY))
      {
        set_security_deny(datum);
      }
      else if (STR_EQ(tag, I_HOSTNAME))
      {
        set_local_hostname(datum);
      }
      else if (STR_EQ(tag, I_PROC_USERID))
      {
        set_process_userid(datum);
      }
      else if (STR_EQ(tag, I_UPDATE_PROG))
      {
        set_update_prog(datum);
      }
      else if (STR_EQ(tag, I_INDEX_PROG))
      {
        set_index_prog(datum);
      }
      else if (STR_EQ(tag, I_CHROOTED))
      { 
        set_chrooted_str(datum);
      }
      else if (STR_EQ(tag, I_DEADMAN))
      {
        set_default_deadman_time(atoi(datum));
      }
      else if (STR_EQ(tag, I_MAX_HITS_CEILING))
      {
        set_max_hits_ceiling (atoi(datum));
      }
      else if (STR_EQ(tag, I_MAX_HITS_DEFAULT))
      {
        set_max_hits_default(atoi(datum));
      }
      else if (STR_EQ(tag, I_PORT))
      {
        set_port(atoi(datum));
      }
      else if (STR_EQ(tag, I_SERVER_TYPE))
      {
        set_server_type_str(datum);
      }
      else if (STR_EQ(tag, I_VERBOSITY))
      {
        set_verbosity(atoi(datum));
      }
      else
      {
        log_error("config file object %s not found", tag); 
      }
    }

    if (fatal)
    {
      log_error( "terminating due to fatal error processing conf. file");
      return FALSE;
    }
  }

  
  fclose(fp);

  return (TRUE);
}

