/* insert std header here */
#include "variables.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <errno.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "client_msgs.h"
#include "compat.h"
#include "conf.h"
#include "defines.h"
#include "fileutils.h"
#include "log.h"
#include "misc.h"
#include "strutil.h"


/* private types */

typedef struct {
  char root_dir[MAX_FILE];          /* !! the default directory !! */
  char bin_path[MAX_FILE];          /* location of binaries */
  char notify_log[MAX_FILE];        /* file path of the -notify log */
  char define_file[MAX_FILE];       /* file path of the format defines */
  char directive_file[MAX_FILE];    /* file path of the X directives */
  char display_file[MAX_FILE];      /* file path of the Top level outputfmts */
  char schema_file[MAX_FILE];       /* file path of the schema file */
  char register_log[MAX_FILE];      /* file path of the -register log */
  char register_spool[MAX_FILE];    /* dir. path for the -register directive */
  char updates_dir[MAX_FILE];       /* dir. path for *something* */
  char root_file[MAX_FILE];         /* file path to the PUNT file */
  char security_allow[MAX_FILE];    /* file path for X directive control */
  char security_deny[MAX_FILE];     /* file path for X directive control */
  char hostname[MAX_LINE];          /* Our FQDN */
  char process_userid[MAX_LINE];    /* Username to setuid to */
  char update_prog[MAX_LINE];       /* Prog to run during -register */
  char index_prog[MAX_LINE];        /* Prog to run during -register to index */
  int  chrooted;                    /* flag indicating chrooted or no */
  int  default_deadman_time;        /* # of secs to be idle */
  int  max_hits_ceiling;            /* can't set -limit beyond this */
  int  max_hits_default;            /* what -limit starts out as */
  int  port;                        /* port we listen to */
  int  root_server;                 /* do we punt? */
  int  server_type;                 /* are we run from inetd, or standalone */
  int  verbose;                     /* what level of logging? */
} config_data_struct;

/* local statics */

/* this is where all of the data contained in the main rwhois
   configuration file (rwhois.conf, normally) is kept */

static config_data_struct   config_data;
static config_data_struct   old_config_data;
static char                 config_file[MAX_FILE];

/* canonicalize_conf_path: a wrapper around canonicalize path that
      fills in much of the required data, and changes the path "in
      place". */
static int
canonicalize_conf_path(path, path_len, is_file, null_allowed)
  char  *path;
  int   path_len;
  int   is_file;
  int   null_allowed;
{
  int   status;
  char  canon_path[MAX_FILE + 1];
  char  full_path[MAX_FILE + 1];
  char  *the_full_path;
  
  status = canonicalize_path(canon_path, sizeof(canon_path), path,
                             config_data.root_dir, is_chrooted(),
                             null_allowed);

  /* if it failed, it failed because the data was bad, or it failed
     the chroot criteria.  We will assume the latter. */
  if (!status)
  {
    log_error("Warning: '%s' isn't in the chroot environment.", path);
    return(status);
  }

  /* canonicalize path will return a "relative to root" path, if it
     can.  Since we don't want to make any assumptions about what the
     cwd is, we will convert it back to a full path */
  if (is_rel_path(canon_path))
  {
    path_rel_to_full(full_path, sizeof(full_path), canon_path,
                     get_root_dir());
    the_full_path = full_path;
  }
  else
  {
    /* the canon_path was already a full path */
    the_full_path = canon_path;
  }
                     
                   
  /* now check the full path to see if it exists */
  if (!null_allowed)
  {
    if (is_file && ! file_exists(the_full_path))
    {
      log_error("Warning: file '%s' does not exist.", the_full_path);
      return FALSE;
    }
    else if (!is_file && ! directory_exists(the_full_path))
    {
      log_error("Warning: directory '%s' does not exist.", the_full_path);
      return FALSE;
    }
  }

  /* copy the canonicalized *relative* path back */
  strncpy(path, canon_path, path_len);

  return TRUE;
}

/* clean and default config data */
void
init_config_data()
{
  /* clear everything */
  bzero(&config_data, sizeof(config_data));
  bzero(config_file, sizeof(config_file));
  
  /* set the compiled defaults */
  set_root_dir(RWHOIS_ROOT_DIR); 
  set_bin_path(DEFAULT_BIN_PATH);
  set_notify_log(DEFAULT_NOTIFY_LOG);
  set_define_file(DEFAULT_DEFINE_FILE);
  set_directive_file(DEFAULT_DIRECTIVE_FILE);
  set_display_file(DEFAULT_DISPLAY_FILE);
  set_schema_file(DEFAULT_SCHEMA_FILE);
  set_register_log(DEFAULT_REGISTER_LOG);
  set_register_spool(DEFAULT_REGISTER_SPOOL);
  set_root_file(DEFAULT_RWHOIS_ROOT_FILE);
  set_security_allow(DEFAULT_SECURITY_ALLOW);
  set_security_deny(DEFAULT_SECURITY_DENY);
  set_local_hostname(sys_gethostname());
  set_update_prog(DEFAULT_UPDATE_PROG);
  set_index_prog(DEFAULT_INDEX_PROG);
  set_chrooted(TRUE);
  set_default_deadman_time(DEFAULT_DEADMAN_TIME);
  set_max_hits_ceiling(CEILING_MAX_HITS);
  set_max_hits_default(DEFAULT_MAX_HITS);
  set_port(DEFAULT_PORT);
  set_server_type(DAEMON_SERVER);
  set_verbosity(1);

  bcopy(&config_data, &old_config_data, sizeof(old_config_data));
}

/* verify_config_data: This is a pass through the config data to
     canonicalize paths, and detect errors that can not be detected
     immediately; it will return TRUE if there were no fatal errors,
     FALSE if not. */
int
verify_config_data()
{
  int   status;
  
  /* if the root directory is "" then we will explictly set it to the CWD */
  if (!*config_data.root_dir)
  {
    char    buf[MAX_FILE + 1];

    getcwd(buf, sizeof(buf));
    set_root_dir(buf);
  }

  /* now we will fix all the paths, and issue errors or warnings */

  status = canonicalize_conf_path(config_data.bin_path,
                                  sizeof(config_data.bin_path),
                                  FALSE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.notify_log,
                                  sizeof(config_data.notify_log),
                                  TRUE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.define_file,
                                  sizeof(config_data.define_file),
                                  TRUE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.directive_file,
                                  sizeof(config_data.directive_file),
                                  TRUE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.display_file,
                                  sizeof(config_data.display_file),
                                  TRUE,
                                  FALSE);

  status = canonicalize_conf_path(config_data.schema_file,
                                  sizeof(config_data.schema_file),
                                  TRUE,
                                  FALSE);

  status = canonicalize_conf_path(config_data.register_log,
                                  sizeof(config_data.register_log),
                                  TRUE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.register_spool,
                                  sizeof(config_data.schema_file),
                                  FALSE,
                                  TRUE);

  status = canonicalize_conf_path(config_data.root_file,
                                  sizeof(config_data.root_file),
                                  TRUE,
                                  FALSE);

  status = canonicalize_conf_path(config_data.security_allow,
                                  sizeof(config_data.security_allow),
                                  TRUE,
                                  FALSE);

  status = canonicalize_conf_path(config_data.security_deny,
                                  sizeof(config_data.security_deny),
                                  TRUE,
                                  FALSE);

  return TRUE;
}

void
display_config_data(file)
  FILE  *file;
{
  fprintf(file, "--------------------------------\n");

  if (*config_data.root_dir) {
    fprintf(file, "root-dir:         %s\n", config_data.root_dir);
  }
  if (*config_data.schema_file) {
    fprintf(file, "schema-file:      %s\n", config_data.schema_file);
  }
  if (*config_data.directive_file) {
    fprintf(file, "directive-file:   %s\n", config_data.directive_file);
  }
  if (*config_data.display_file) {
    fprintf(file, "display-file:     %s\n", config_data.display_file);
  }
  if (*config_data.define_file) {
    fprintf(file, "define-file:      %s\n", config_data.define_file);
  }
  if (*config_data.register_log) {
    fprintf(file, "register-log:     %s\n", config_data.register_log);
  }
  if (*config_data.register_spool) {
    fprintf(file, "register-spool:   %s\n",
            config_data.register_spool);
  }
  if (*config_data.notify_log) {
    fprintf(file, "notify-log:       %s\n", config_data.notify_log);
  }
  if (*config_data.root_file) {
    fprintf(file, "root-file:        %s\n", config_data.root_file);
  }
  if (*config_data.security_allow) {
    fprintf(file, "security-allow:   %s\n",
            config_data.security_allow);
  }
  if (*config_data.security_deny) {
    fprintf(file, "security-deny:    %s\n",
    config_data.security_deny);
  }
  if (*config_data.bin_path) {
    fprintf(file, "bin-path:         %s\n", config_data.bin_path);
  }
  if (*config_data.update_prog) {
    fprintf(file, "update-prog:      %s\n", config_data.update_prog);
  }
  if (*config_data.index_prog) {
    fprintf(file, "index-prog:       %s\n", config_data.index_prog);
  }
  if (*config_data.hostname) {
    fprintf(file, "local-host:       %s\n", config_data.hostname);
  }

  fprintf(file, "local-port:       %d\n", config_data.port);

  if (*config_data.process_userid) {
    fprintf(file, "userid:           %s\n",
            config_data.process_userid);
  }

  fprintf(file, "max-hits-ceiling: %d\n", config_data.max_hits_ceiling);
  fprintf(file, "max-hits-default: %d\n", config_data.max_hits_default);

  fprintf(file, "chrooted:         %s\n",
          (config_data.chrooted ? "YES" : "NO"));

  fprintf(file, "server-type:      %s\n",
          (config_data.server_type ? "INETD" : "DAEMON"));
  
  fprintf(file, "verbosity-level:  %d\n", config_data.verbose);
  
  fprintf(file, "--------------------------------\n");        
}
/* the guard functions */

/* set_root_dir: We can actually detect errors on this one. */
int
set_root_dir(dir)
  char  *dir;
{
  if (dir && *dir)
  {
    if (directory_exists(dir))
    {
      strncpy(config_data.root_dir, dir, MAX_FILE);
      
      /* strip leading and trailing spaces, and strip trailing '/' */
      trim(config_data.root_dir);
      if (strlen(dir) > 1)
      {
        strip_trailing(config_data.root_dir, '/');
      }
    }
    else
    {
      log_error("root directory '%s' does not exist.", dir);
      return FALSE;
    }
  }

  return TRUE;
}

char *
get_root_dir()
{
  return(config_data.root_dir);
}

int
chdir_root_dir()
{
  char  *dir = get_root_dir();
  
  /* attempt to set the working directory to the root dir */
  if (chdir(dir) < 0)
  {
    log_error("chdir to '%s' failed: %s", dir, l_strerror(errno));
    return FALSE;
  }

  return TRUE;
}

int
set_bin_path(path)
  char  *path;
{
  strncpy(config_data.bin_path, path, MAX_FILE);
  return TRUE;
}

char *
get_bin_path()
{
  return(config_data.bin_path);
}

int
set_notify_log(log)
  char  *log;
{
  strncpy(config_data.notify_log, log, MAX_FILE);
  return TRUE;
}

char *
get_notify_log()
{
  return(config_data.notify_log);
}

int
set_define_file(file)
  char  *file;
{
  strncpy(config_data.define_file, file, MAX_FILE);
  return TRUE;
}

char *
get_define_file()
{
  return(config_data.define_file);
}

int
set_directive_file(file)
  char *file;
{
  strncpy(config_data.directive_file, file, MAX_FILE);
  return TRUE;
}

char *
get_directive_file()
{
  return(config_data.directive_file);
}

int
set_display_file(file)
  char  *file;
{
  strncpy(config_data.display_file, file, MAX_FILE);
  return TRUE;
}

char *
get_display_file()
{
  return(config_data.display_file);
}

int
set_schema_file(file)
  char  *file;
{
  strncpy(config_data.schema_file, file, MAX_FILE);
  return TRUE;
}

char *
get_schema_file()
{
  return(config_data.schema_file);
}

int
set_register_log(log)
  char  *log;
{
  strncpy(config_data.register_log, log, MAX_FILE);
  return TRUE;
}

char *
get_register_log()
{
  return(config_data.register_log);
}

int
set_register_spool(spool)
  char  *spool;
{
  strncpy(config_data.register_spool, spool, MAX_FILE);
  return TRUE;
}

char *
get_register_spool()
{
  return(config_data.register_spool);
}

int
set_updates_dir(dir)
  char *dir;
{
  strncpy(config_data.updates_dir, dir, MAX_FILE);
  return TRUE;
}

char *
get_updates_dir()
{
  return(config_data.updates_dir);
}

int
set_root_file(file)
  char  *file;
{
  strncpy(config_data.root_file, file, MAX_FILE);
  return TRUE;
}

char *
get_root_file()
{
  return(config_data.root_file);
}

int
set_security_allow(file)
  char  *file;
{
  strncpy(config_data.security_allow, file, MAX_FILE);
  return TRUE;
}

char *
get_security_allow()
{
  return(config_data.security_allow);
}

int
set_security_deny(file)
  char  *file;
{
  strncpy(config_data.security_deny, file, MAX_FILE);
  return TRUE;
}

char *
get_security_deny()
{
  return(config_data.security_deny);
}

int
set_local_hostname(name)
  char  *name;
{
  strncpy(config_data.hostname, name, MAX_LINE);
  return TRUE;
}

char *
get_local_hostname()
{
  return(config_data.hostname);
}

int
set_process_userid(id)
  char  *id;
{
  strncpy(config_data.process_userid, id, MAX_LINE);
  return TRUE;
}

char *
get_process_userid()
{
  return(config_data.process_userid);
}

int
set_update_prog(prog)
  char  *prog;
{
  strncpy(config_data.update_prog, prog, MAX_LINE);
  return TRUE;
}

char *
get_update_prog()
{
  return(config_data.update_prog);
}

int
set_index_prog(prog)
  char  *prog;
{
  strncpy(config_data.index_prog, prog, MAX_FILE);
  return TRUE;
}

char *
get_index_prog()
{
  return(config_data.index_prog);
}

int
set_chrooted(val)
  int   val;
{
  config_data.chrooted = val;
  return TRUE;
}

int
set_chrooted_str(str)
  char  *str;
{
  config_data.chrooted = true_false(str);
  return TRUE;
}

int
is_chrooted()
{
  return(config_data.chrooted);
}

int
set_default_deadman_time(sec)
  int   sec;
{
  config_data.default_deadman_time = sec;
  return TRUE;
}

int
get_default_deadman_time()
{
  return(config_data.default_deadman_time);
}

int
set_max_hits_ceiling(hits)
  int   hits;
{
  config_data.max_hits_ceiling = hits;
  return TRUE;
}

int
get_max_hits_ceiling()
{
  return(config_data.max_hits_ceiling);
}

int
set_max_hits_default(hits)
  int   hits;
{
  config_data.max_hits_default = hits;
  return TRUE;
}

int
get_max_hits_default()
{
  return(config_data.max_hits_default);
}

int
set_port(p)
  int   p;
{
  config_data.port = p;
  return TRUE;
}

int
get_port()
{
  return(config_data.port);
}

int
set_root_server(val)
  int   val;
{
  config_data.root_server = val;
  return TRUE;
}

int
set_root_server_str(str)
  char  *str;
{
  config_data.root_server = true_false(str);
  return TRUE;
}

int
is_root_server()
{
  return(config_data.root_server);
}

int
set_server_type(type)
  int   type;
{
  config_data.server_type = type;
  return TRUE;
}

int
set_server_type_str(str)
  char *str;

{
  if (STR_EQ(str, "INETD") ) {
    config_data.server_type = INETD_SERVER;
  }
  else {
    config_data.server_type = DAEMON_SERVER;
  }

  return TRUE;
}

int
get_server_type()
{
  return(config_data.server_type);
}

/* is_daemon_server: true if DAEMON, false if not */
int
is_daemon_server()
{
  if (get_server_type() == DAEMON_SERVER) {
    return (TRUE);
  }
  return (FALSE);
}


int
set_verbosity(val)
  int   val;
{
  config_data.verbose = val;
  return TRUE;
}

int
get_verbosity()
{
  return config_data.verbose;
}

int
set_config_file(file)
  char *file;
{
  strncpy(config_file, file, sizeof(config_file));

  return TRUE;
}

char *
get_config_file()
{
  return(config_file);
}
