/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "reduce.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "defines.h"
#include "misc.h"
#include "parse.h"
#include "strutil.h"
#include "whois.h"


#define CHECK       1
#define REDUCE_IT   2

/**
 local prototypes
***/
static int is_email_address PROTO((query_item_struct *qitem, int pass,
                                   int mode));

static void bump_off_userid PROTO((char *str));

static int is_domain PROTO((query_item_struct *qitem, int pass, int mode));

static void bump_off_domain PROTO((char *str));

static void bump_off_net_prefix PROTO((char *str));

static char *convert_to_ip_net_format PROTO((char *ip_str, int prefix_len));

static int parse_for_netlen PROTO((char *addrstr));

static int strip_netlen PROTO((char *str));

static unsigned long addtolong PROTO((char *addstr));

static char *longtoaddstr PROTO((unsigned long t, char *buf));

static unsigned long mask_addr_to_len PROTO((unsigned long addr, int len));

/******************************************************************************
  is the query reducible
  returns TRUE if can make reduction 
          FALSE if not
******************************************************************************/
int
is_query_reducible(query_list, pass)
  query_item_struct *query_list;
  int               pass;

{
  int   retval = -1;
  char  *c;

  while (query_list != NULL)
  {
    c = query_list->value;

    while (*c != '\0')
    {
      if (!mak_isalnum(*c++))
      {
        *query_list->value = '\0';
      }
    }

    if (is_email_address(query_list, pass, CHECK))
    {
      retval = KEEPON_ACTION;
    }
    else if (is_ip_network(query_list, pass, CHECK))
    {
      retval = KEEPON_ACTION;
    }
    else if (is_domain(query_list, pass, CHECK))
    {
      retval = KEEPON_ACTION;
    }

    query_list=query_list->nxt;
  }

  if (pass == 1 && retval == -1)
  {
    return (WHOIS_PP_ACTION);
  }
  else if (retval == -1)
  {
    return (PUNT_ACTION);
  }

  return (retval);
}

/******************************************************************************
  this procedure modifies the query to get a referral
  does not make a reduction on pass 1
  returns TRUE if reduction made
          FALSE if not
******************************************************************************/
int
reduce_query (query_list, pass)
  query_item_struct *query_list;
  int               pass;

{
  int reduction_made = FALSE;

  while (query_list != NULL)
  {
    if (is_email_address(query_list, pass, REDUCE_IT) && pass > 1)
    {
      bump_off_userid(query_list->value);
      query_list->attribute = NEW_STRING ("Domain-Name");
      reduction_made = TRUE;
    }
    else if (is_ip_network(query_list, pass, REDUCE_IT))
    {
      if (pass > 1)
      {
        bump_off_net_prefix(query_list->value);
      }
      query_list->attribute = NEW_STRING ("IP-Network");
      reduction_made = TRUE;
    }
    else if (is_domain(query_list, pass, REDUCE_IT))
    {
      if (pass > 1)
      {
        bump_off_domain (query_list->value);
      }
      query_list->attribute = NEW_STRING ("Domain-Name");
      reduction_made = TRUE;
    }

    query_list=query_list->nxt;
  }

  return (reduction_made);
}

/******************************************************************************
  this function checks to see if it is a email address
  returns TRUE if can reduce query
          FALSE if can not
******************************************************************************/
static int
is_email_address (qitem, pass, mode)
  query_item_struct *qitem;
  int               pass;
  int               mode;
{
  if (strchr (qitem->value, '@'))
  {
    return (TRUE);
  }

  return (FALSE);
}

/******************************************************************************
  this procedure strips off the userid on a rfc822 email address
******************************************************************************/
static void
bump_off_userid (str)
  char *str;
{
  char buf[MAX_LINE];

  strcpy(buf, strchr(str,'@') + 1);
  strcpy(str, buf);
}

/******************************************************************************
  this function checks to see if it is a domain name
  returns TRUE if is a domain name
          FALSE if not
******************************************************************************/
static int
is_domain (qitem, pass, mode)
  query_item_struct *qitem;
  int               pass;
  int               mode;

{
  char *c = qitem->value;

  if (strchr(qitem->value, '.') == NULL)
  {
    if (strlen(qitem->value) != 2 || (qitem->loop_detect && mode == CHECK) )
    {
      return (FALSE);
    }
    else if (strlen(qitem->value) == 2)
    { 
      return (FALSE);
    }
  }
  else if (strlen(strchr (qitem->value, '.')) == 1 && 
           strlen(qitem->value) > 3)
  {
    return (FALSE);
  }
  else if (qitem->loop_detect && mode == CHECK)
  {
    return (FALSE);
  }

  while ( *c != '\0' && (isalnum(*c) || *c == '.' || *c == '-'))
  {
    c++;
  }
  if (*c == '\0')
  {
    return (TRUE);
  }
  return (FALSE);
}

/******************************************************************************
  this procedure bumps off the last octet if not found
******************************************************************************/
static void
bump_off_domain(str)
  char *str;
{
  /** 
    for top-level stuff 
    ***/
  if (strlen (str) == 3 && str[strlen(str)-1] == '.')
  {
    return;
  }

  strrev(str);
  *strrchr(str,'.') = '\0';
  strrev(str);
}

/******************************************************************************
  this function checks to see if it is a ip number
  returns TRUE if is a ip number
          FALSE if not
******************************************************************************/
int
is_ip_network(qitem, pass, mode)
  query_item_struct *qitem;
  int               pass;
  int               mode;
{
  char  *c = qitem->value;
  int   prefix;
  char  *new_val;
  char  *tmp_val;
  int   punc_found = FALSE;
  
  /* rule out empty strings */
  if (!c || !*c)
  {
    return FALSE;
  }
  
  /* advance through digits, dots, and slashes */
  while ( *c != '\0')
  {
    if (!isdigit(*c) && *c != '.' && *c != '/')
    {
      return FALSE;
    }
    else if (*c == '.' || *c == '/')
    {
      punc_found = TRUE;
    }
    c++;
  }

  /* if we didn't have one of the punc marks, it was just a raw
     number.  Who knows what that is? */
  if (!punc_found)
  {
    return FALSE;
  }
  
  /* put the ip-network string into a standard format */
  tmp_val = convert_to_ip_net_format(qitem->value, 0);
  if (tmp_val)
  {
    new_val = calloc(strlen(tmp_val), 1);
    strcpy(new_val, tmp_val);
    free(qitem->value);
    qitem->value = new_val;
    if (debug_status())
    {
      printf("%debug: reformatted query item to %s\n", new_val);
    }
  }
  
  prefix = parse_for_netlen(qitem->value);

  if (prefix <= 8)
  {
    if (qitem->loop_detect == TRUE && mode == CHECK)
    {
      return (FALSE);
    }
    else
    {
      qitem->loop_detect = TRUE;
    }
  }

  return (TRUE);
}

/******************************************************************************
  this procedure bumps off the last prefix if not found
  if prefix already three
******************************************************************************/
static void
bump_off_net_prefix(str)
  char *str;

{
  char  *tmp_ip;
  int   netlen = parse_for_netlen (str);

  if (!strip_netlen (str))
  {
    netlen = 24;
  }
  else if (netlen > 24)
  {
    netlen = 24;
  }
  else if (netlen > 8)
  {
    netlen--;
  }

  tmp_ip = convert_to_ip_net_format(str, netlen);
  strcpy(str, tmp_ip);
}

static char *
convert_to_ip_net_format(ip_str, prefix_len)
  char *ip_str;
  int  prefix_len;
{
  unsigned long raw_ip      = 0;
  static char   tmp_ip[25];
  char          tmp_netlen[5];
  
  if (!ip_str || !*ip_str)
  {
    return NULL;
  }
  
  if (!prefix_len)
  {
    /* calculate the prefix len either from the /preflen or the number
       of octets */
    prefix_len = parse_for_netlen(ip_str);
  }

  /* chop of the network length indicator */
  strip_netlen(ip_str);
  /* convert the octet part to an integer */
  raw_ip = addtolong(ip_str);

  /* mask off the insignificant bits */
  raw_ip = mask_addr_to_len(raw_ip, prefix_len);

  /* convert back to a string */
  longtoaddstr(raw_ip, tmp_ip);

  sprintf(tmp_netlen, "/%d", prefix_len);

  strcat(tmp_ip, tmp_netlen);

  return(tmp_ip);
}

/***********************************************************************
  parses network address to length
************************************************************************/
static int
parse_for_netlen(addrstr)
  char *addrstr;
{
  int   o1  = 0;
  int   o2  = 0;
  int   o3  = 0;
  int   o4  = 0;
  int   octsret;

  /* if there is a '/', presume it is followed by a network length
     indicator, and return it. */
  if (rindex(addrstr, '/') != NULL)
  {
    return (atoi(rindex(addrstr, '/') + 1));
  }

  /* otherwise, we read in the four octets */
  octsret = sscanf(addrstr, "%d.%d.%d.%d", &o1, &o2, &o3, &o4); 
  if (octsret == 4 || octsret == 3)
  {
    return (24);
  }
  else if (octsret == 2)
  {
    return (16);
  }

  return (8);
}

/***********************************************************************
  strips the netlen
************************************************************************/
static int
strip_netlen(str)
  char *str;
{
  if (rindex (str, '/') != NULL)
  {
    *strrchr (str, '/') = '\0';
    return (TRUE);
  }
  return (FALSE);
}


/***********************************************************************
  Address Convert from str to long
************************************************************************/
static unsigned long
addtolong (addstr)
  char *addstr;
{
  unsigned long addr;
  int           o1 = 0;
  int           o2 = 0;
  int           o3 = 0;
  int           o4 = 0;
  int           octsret;

  octsret = sscanf(addstr, "%d.%d.%d.%d", &o1, &o2, &o3, &o4);
  
  if (octsret < 4)
  {
    o4 = 0;
  }
  if (octsret < 3)
  {
    o3 = 0;
  }
  if (octsret < 2)
  {
    o2 = 0;
  }
    
  addr = (((unsigned)o1 & 0xFF) << 24) + (((unsigned)o2 & 0xFF) << 16) +
         (((unsigned)o3 & 0xFF) << 8) + ((unsigned) o4 & 0xFF);

  return (addr);
}

/***********************************************************************
  Address Convert from long to str
************************************************************************/
static char *
longtoaddstr(t, buf)
  unsigned long t;
  char          *buf;

{
  sprintf(buf,
          "%d.%d.%d.%d",
          (unsigned int) t >> 24,
          (unsigned int) (t >> 16) & 0XFF, 
          (unsigned int) (t >> 8) & 0xFF,
          (unsigned int) t & 0xFF);
  return (buf);
}

/***********************************************************************
  Mask addr to prefix
************************************************************************/
static unsigned long
mask_addr_to_len(addr, len)
  unsigned long addr;
  int           len;

{
  unsigned long mask = 0xffffffff;

  mask = mask << (32-len);
  return (addr&mask);
}
