/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "io.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#include <sys/stat.h>

#include "llist.h"
#include "log.h"
#include "objects.h"
#include "variables.h"


/* local definitions */

#define DB_BASE_TEMPLATE        "%s.db"
#define DB_UNSORTED_TEMPLATE    "%s-%d.sort"
#define DB_INDEX_TEMPLATE       "%s-%d.ndx"
#define DATA_FILE               "DATA"
#define INDEX_FILE              "INDEX"

/* local statics/global */

static int      ensure_in_jail;
llist_struct    *open_object_db_list = NULL;


/* public functions */

void
set_ensure_in_jail(flag)
  int   flag;
{
  ensure_in_jail = flag;
}

int
get_ensure_in_jail()
{
  return(ensure_in_jail);
}

/* get_object_db: opens the index files; expects to have the rwhois
   root as cwd. */
FILE *
get_object_db(object_name, data_file, mode, type, file_no)
  char  *object_name;
  char  *data_file;
  int   mode;
  int   type;
  int   *file_no;
{
  object_struct    *object;
  rwhois_io_struct *ptr;
  llist_struct     *cur         = open_object_db_list;
  char             *base_name;
  int              status;
  
  /* Sees if the object info is already open.  If it is and same
     filename - cool, else closes out the filename and starts a new
     one with a new file_no. */

  while (cur != NULL)
  {
    ptr = cur->datum;

    if (STR_EQ(ptr->object_name, object_name))
    {
      /* got a new file - save info on old one */
      if (strcmp(data_file, ptr->data_file))
      {
        save_in_db(ptr->base_fp, ptr->data_file, DATA_FILE, ptr->file_no,
                   ptr->num_recs);
        strcpy(ptr->data_file, data_file);
        ptr->num_recs = 0;
        ptr->file_no++;
      }
      *file_no = ptr->file_no;
      return(ptr->index_fp);
    }
    cur = cur->nxt;
  }

  /* object info not already open */
  
  if ((object = get_object_by_object_name(object_name)) == NULL)
  {
    log_error("open_object_db: can not find object '%s'", 
              object_name);
    return NULL;
  }

  ptr             = NEW_RWHOIS_IO;
  strcpy(ptr->object_name, object_name);
  strcpy(ptr->data_file, data_file);
  ptr->num_recs   = 0;
  ptr->total_recs = 0;

  if (type == AUTHORITATIVE)
  {
    base_name = object->local_db;
  }
  else
  {
    base_name = object->cache_db;
  }

  if (mode == INITIALIZE)
  {
    status = initialize_rwhois_db(base_name, ptr);
    if (!status)
    {
      log_error("get_object_db: database initialization failed\n");
      return NULL;
    }
  }
  else
  {
    status = append_rwhois_db(base_name, ptr);
    if (!status)
    {
      log_error("get_object_db: database append setup failed\n");
      return NULL;
    }
  }

  *file_no = ptr->file_no;

  if (ptr->base_fp != NULL)
  {
    open_object_db_list = append_llist(open_object_db_list, ptr);
  }

  return(ptr->index_fp);
}

/* initialize_rwhois_db: initializes the database */
int
initialize_rwhois_db(base_name, ptr)
  char              *base_name;
  rwhois_io_struct  *ptr;

{
  char basefname[MAX_FILENAME];

  sprintf(basefname, DB_BASE_TEMPLATE, base_name);

  clear_existing_indexes(basefname);

  if ((ptr->base_fp = fopen(basefname, "w")) == NULL)
  {
    log_error("could not create '%s': %s", basefname, l_strerror(errno));
    return FALSE;
  }

  sprintf(ptr->index_unsorted, DB_UNSORTED_TEMPLATE, base_name, 0);
  sprintf(ptr->index_sorted, DB_INDEX_TEMPLATE, base_name, 0);

  if ((ptr->index_fp = fopen(ptr->index_unsorted, "w")) == NULL)
  {
    log_error("could not create '%s': %s", ptr->index_unsorted,
              l_strerror(errno));
    return FALSE;
  }

  return TRUE;
}

/* append_rwhois_db: appends info to the database */
int
append_rwhois_db(base_name, ptr)
  char              *base_name;
  rwhois_io_struct  *ptr;
{
  char                  basefname[MAX_FILENAME];
  mkdb_fileinfo_struct  *fileinfo = NULL;
  int                   num_indexes;

  sprintf(basefname, DB_BASE_TEMPLATE, base_name);

  if ((ptr->base_fp = fopen(basefname, "r+")) != NULL)
  {
    fileinfo = load_fileinfo_list(ptr->base_fp);
  }
  else
  {
    log_error("could not open '%s': %s", basefname, l_strerror(errno));
    return FALSE;
  }

  ptr->file_no = next_file_no_from_fileinfo(fileinfo);
  num_indexes  = next_index_no_from_fileinfo(fileinfo);

  sprintf(ptr->index_unsorted, DB_UNSORTED_TEMPLATE, base_name, num_indexes);
  sprintf(ptr->index_sorted, DB_INDEX_TEMPLATE, base_name, num_indexes);

  if ((ptr->index_fp = fopen(ptr->index_unsorted, "w")) == NULL)
  {
    log_error("could not create/open '%s': %s", ptr->index_unsorted,
              l_strerror(errno));
    return FALSE;
  }
  
  clear_fileinfo_list(fileinfo);

  return TRUE;
}


/* next_file_no_from_fileinfo: sets the next fileinfo number from the
   list returns the next available number */
int
next_file_no_from_fileinfo(fileinfo)
  mkdb_fileinfo_struct *fileinfo;
{
  int max = 0;

  while (fileinfo != NULL)
  {
    if (fileinfo->index_no > max)
    {
      max = fileinfo->index_no;
    }
    fileinfo = fileinfo->nxt;
  }
  return(max + 1);
}

/* next_index_no_from_fileinfo: returns the next available index number */
int
next_index_no_from_fileinfo(fileinfo)
  mkdb_fileinfo_struct *fileinfo;
{
  int cnt = 0;

  while (fileinfo != NULL)
  {
    if (fileinfo->type == ENUM_INDEX_FILE)
    {
      cnt++;
    }
    fileinfo = fileinfo->nxt;
  }
  return(cnt);
}

/* clear_existing_indexes: removes any existing indexes */
void
clear_existing_indexes (basefname)
  char *basefname;
{
  FILE                  *fp;
  mkdb_fileinfo_struct  *fileinfo;
  mkdb_fileinfo_struct  *cur;

  /* open the fileinfo directory ("local.db" et. al.) */
  if ((fp = fopen(basefname, "r")) != NULL)
  {
    cur = fileinfo = load_fileinfo_list(fp);

    /* remove all of the index files */
    while (cur != NULL)
    {
      if (cur->type == ENUM_INDEX_FILE)
      {
        unlink(cur->filename);
      }
      cur = cur->nxt;
    }
    clear_fileinfo_list(fileinfo);
    fclose(fp);

    /* remove the fileinfo directory itself */
    unlink(basefname);
  }
}

/* save_in_db: saves file info into db */
void
save_in_db(fp, data_file, type, file_no, num_recs)
  FILE  *fp;

  char  *data_file;
  char  *type;
  int   file_no;
  int   num_recs;
{
  struct stat   file_info;
  
  stat(data_file, &file_info);

  if (file_no > 0)
  {
    fprintf(fp, "---\n");
  }

  fprintf(fp, "%s: %s\n", MKDB_TYPE, type);
  fprintf(fp, "%s: %s\n", MKDB_FILE, data_file);
  fprintf(fp, "%s: %d\n", MKDB_INDEX_NO, file_no);
  fprintf(fp, "%s: %d\n", MKDB_NUMRECS, num_recs);
  fprintf(fp, "%s: %d\n", MKDB_SIZE, (int) file_info.st_size);
}

/* increment_record_count: increments the record count based on name
   and data file */
void
increment_record_count (object_name, data_file)
  char *object_name;
  char *data_file;
{
  rwhois_io_struct  *ptr = NULL;
  llist_struct      *cur = open_object_db_list;

  while (cur != NULL)
  {
    ptr = cur->datum;
    if ( (STR_EQ(ptr->object_name, object_name)) && 
         (!strcmp(data_file, ptr->data_file)) )  {
      ptr->num_recs++;
      ptr->total_recs++;
      return;
    }
    cur = cur->nxt;
  }
}

/* flush_and_sort: closes the existing data files, sorts, and creates
   a index record */
int
flush_and_sort(in_jail)
  int in_jail;
{
  rwhois_io_struct  *ptr = NULL;
  llist_struct      *cur = open_object_db_list;

  while (cur != NULL)
  {
    ptr = cur->datum;
    /* save in the data file stuff */
    save_in_db(ptr->base_fp, ptr->data_file, DATA_FILE, ptr->file_no,
               ptr->num_recs); 

    /* close the index file, and sort it */
    fclose(ptr->index_fp);

    if (!sort_index(in_jail, ptr->index_unsorted, ptr->index_sorted))
    {
      return FALSE;
    }

    save_in_db(ptr->base_fp, ptr->index_sorted, INDEX_FILE, ptr->file_no+1,
               ptr->total_recs);

    /* close the db file since on to the next object */
    fclose(ptr->base_fp);
    cur = cur->nxt;
  }

  return TRUE;
}

/* sort_index: sorts the index file */
int
sort_index(in_jail, unsorted_name, sorted_name)
  int   in_jail;
  char  *unsorted_name;
  char  *sorted_name;
{
  char buf[MAX_BUF];
  
  if (in_jail)
  {
    sprintf(buf,"/bin/sort -i +3 -t':' %s -o %s", unsorted_name,
            sorted_name);
  }
  else
  {
    sprintf(buf,"/usr/bin/sort -i +3 -t':' %s -o %s", unsorted_name, 
            sorted_name);
  }
  if (get_verbosity() > 2)
  {
    printf("sorting: %s ...", buf);
  }
  else if (get_verbosity())
  {
    printf("sorting %s ...", sorted_name);
  }


  if (system(buf) < 0 )
  { 
    log_error("sort failed: %s", l_strerror(errno));
    return FALSE;
  }
  
  unlink(unsorted_name);

  if (get_verbosity())
  {
    printf ("done.\n");
  }

  return TRUE;
}





