/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
  Stan Borinski
***************************************************************************/
#include "parse.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "database.h"
#include "defines.h"
#include "fileinfo.h"
#include "client_msgs.h"
#include "conf.h"
#include "llist.h"
#include "log.h"
#include "misc.h"
#include "read_config.h"
#include "strutil.h"
#include "validate.h"
#include "variables.h"


#define OUTPUT_FILE_FORMAT   "%s.txt"



/* local globals */
int     in_jail         = FALSE;
char    *config_file    = NULL;

/* local prototypes */

static void usage PROTO((char *prog_name));

static int set_header_info PROTO((FILE *ifp, FILE *ofp, char *version,
                                  char *email, char *action, int *row_no));

static void debug_record_list PROTO((llist_struct *rec_list));

static void debug_record PROTO((llist_struct *att));

void
main(argc, argv)
  int   argc;
  char  *argv[];
{
  extern int    optind;
  extern char   *optarg;
  char          c;
  int           no_err          = TRUE;
  char          *path;
  char          email[MAX_LINE];
  char          action[MAX_LINE];
  llist_struct  *record_list;

  /* initialization */

  /* command line parsing */

  setbuf(stdout, NULL);

  while ((c = getopt(argc, argv, "jqc:")) != EOF)
  {
    switch (c)
    {
    case 'c':
      config_file = optarg;
      break;
    case 'j':
      in_jail = TRUE;
      break;
    case 'q':
      set_verbosity(0);
      break;
    }
  }
  if (argc == 1)
  {
    usage(argv[0]);
  }

  /* read in rwhois configuration items */

  if (config_file == NULL) {
    config_file = DEFAULT_RWHOIS_CONFIG_FILE;
  }
  
  if (!read_schema_config_files(config_file, in_jail))
  {
    log_error("rwhoisparse: problem with config files");
    exit (1);
  }

  chdir_root_dir();
  
  while (optind < argc && no_err)
  {
    path    = argv[optind];
    
    if ((no_err = parse_file(path, &record_list, action,
                             email)))
    {
      no_err = update_database(action, record_list, path);
    }

    optind++;
  } 
}

static void
usage(prog_name)
  char  *prog_name;
{
  fprintf(stderr, "usage: %s [-c configfile] [-j] [-q] files...\n", prog_name);
  fprintf(stderr, "       -c configfile: location of the rwhois conf. file\n");
  fprintf(stderr, "       -j: in jail; program running in a chrooted env.\n");
  fprintf(stderr, "       -q: quiet; suppress error/info messages.\n");
  exit(64);
}

/*************************************************************************
 parses the file
 RETURNS  TRUE if ok
          FALSE if not
**************************************************************************/
int
parse_file(data_file, top, action, email)
  char          *data_file;
  llist_struct  **top;
  char          *action;
  char          *email;

{
  char          output_file[MAX_LINE];
  char          line[MAX_LINE];
  char          version[MAX_LINE];
  llist_struct  *record_list            = NULL;
  llist_struct  *record                 = NULL;
  line_struct   *new;
  FILE          *ifp;
  FILE          *ofp;
  int           row_no                  = 0;
  int           retval                  = TRUE;

  /* first read in record */

  if ((ifp = fopen(data_file, "r")) == NULL)
  {
    fprintf(stderr, "parse_file: can not open %s\n", data_file);
    return(FALSE);
  }

  sprintf(output_file, OUTPUT_FILE_FORMAT, data_file);

  if ((ofp = fopen(output_file, "w")) == NULL)
  {
    fprintf(stderr, "parse_file: can not open %s\n", data_file);
    return(FALSE);
  }

  if (!set_header_info(ifp, ofp, version, email, action, &row_no))
  {
    fprintf(stderr, "parse_file: can not do header!\n");
    fclose(ifp);
    fclose(ofp);
    return(FALSE);
  }

  while (fgets(line, MAX_LINE, ifp) != NULL)
  {
    row_no++;
    fputs(line, ofp);
    stripchar(line, '\n');
    trim(line);
    if (!*line)
    {
      if (record != NULL)
      {
        record_list = append_llist (record_list, record);
      }
      record = NULL;
    }
    else
    {
      new         = NEW_LINE_STRUCT;
      new->row_no = row_no;
      new->line   = NEW_STRING (line);
      record      = append_llist (record, new);
    }
  }
  fclose (ifp);

  /***
    if no end of record sent before EOF
    ****/

  if (record != NULL)
  {
    record_list = append_llist (record_list, record);
  }

  /*******
    now validate record
    *******/
  
  *top = record_list;
  while (record_list != NULL)
  {
    if (!validate_record (ifp, ofp, record_list->datum))
    {
      /* fprintf (stderr, "parse_file: bombed on validate_record\n"); */
      retval = FALSE;
    }
    record_list = record_list->nxt;
  }
  fclose (ofp);

  /*******
    print out the errors if found
    ********/

  unlink (output_file);

  /*
    debug_record_list (record_list);
    debug_record_list (record_list);
    */
  return (retval);
}

/*************************************************************************
sets the version #, email address, and action desired
**************************************************************************/
static int
set_header_info (ifp, ofp, version, email, action, row_no)
  FILE  *ifp;
  FILE  *ofp;
  char  *version;
  char  *email;
  char  *action;
  int   *row_no;
{
  char line[MAX_LINE];
  char tag[MAX_LINE];
  char datum[MAX_LINE];
  int  i;

  *version = '\0';
  *email   = '\0';
  *action  = '\0';

  for (i = 0; i < 3; i++)
  {
    (*row_no)++;
    if (fgets(line, MAX_LINE, ifp) != NULL)
    {
      fputs(line, ofp);
      stripchar(line, '\n');
      if (parse_line(line, tag, datum))
      {
        if (STR_EQ(tag, H_VERSION))
        {
          strcpy(version, datum);
        }
        else if (STR_EQ(tag, H_EMAIL))
        {
          strcpy (email, datum);
        }
        else if (STR_EQ(tag, H_ACTION))
        {
          strcpy (action, datum);
        }
      }
    }
    else
    {
      print_error_to_file(ifp, ofp, TRUE, *row_no, 
                          "Undefined Header tag '%s'\n", tag);
      return(FALSE);
    }
  } 
  if ( !*version || !*email || !*action)
  {
    print_error_to_file(ifp, ofp, TRUE, *row_no, "Required Header missing");
    return(FALSE);
  }
  return(TRUE);
}


static void
debug_record_list(rec_list)
  llist_struct *rec_list;
{
  printf("debug_record: on!\n");
  while (rec_list != NULL)
  {
    debug_record(rec_list->datum);
    rec_list = rec_list->nxt;
  }
  printf("debug_record: done!\n");
}

static void
debug_record(att)
  llist_struct *att;
{
  line_struct *line;

  while (att!=NULL)
  {
    line = att->datum;
    printf ("%d: %s\n", line->row_no, line->line);
    att=att->nxt;
  }
}

